@@ -5,7 +5,7 @@
 import os
 import tempfile
 
-__version__ = '2.41.2'
+__version__ = '2.51.1'
 
 __TEMP_STORAGE__ = os.path.join(tempfile.gettempdir(), 'carolina/cache')
 
@@ -20,6 +20,10 @@ _CAROL_METADATA_GOLDEN = ['mdmCounterForEntity', 'mdmStagingCounter', 'mdmId', '
                           'mdmApplicationIdMasterRecordId',
                           'mdmPreviousIds', 'mdmDeleted']
 
+_CAROL_METADATA_UNTIE_GOLDEN = 'mdmStagingCounter'
+_CAROL_METADATA_UNTIE_STAGING = 'mdmCounterForEntity'
+
+
 _NEEDED_FOR_MERGE = ['mdmCounterForEntity', 'mdmId', 'mdmDeleted']
 
 _REJECTED_DM_COLS = ['mdmConnectorId', 'mdmCreated', 'mdmCreatedUser', 'mdmCrosswalk',
@@ -43,3 +47,6 @@ from .auth.PwdKeyAuth import PwdKeyAuth
 from .cds import CDSGolden, CDSStaging
 from .apps import Apps
 from .subscription import Subscription
+from .carol_api import CarolAPI
+from . import bigquery
+from .sql import SQL
\ No newline at end of file
