@@ -1,3 +1,4 @@
+import warnings
 from urllib3.util.retry import Retry
 import requests
 from requests.adapters import HTTPAdapter
@@ -14,7 +15,7 @@ from .exceptions import CarolApiResponseException, InvalidToken
 
 class Carol:
     """
-    This class handle all Carol`s API calls It will handle all API calls,
+    Handles all Carol`s API calls It will handle all API calls,
     for a given authentication method. :param domain: `str`.
 
     Args:
@@ -47,6 +48,15 @@ class Carol:
                 2. else host={organization}.{environment}
 
             See Carol._set_host.
+        
+         user:  `str` default `None`
+            User
+         password: `str` default `None`
+            User passowrd
+         api_key:  `str` default `None`
+            Carol's Api Key
+        org_level: `bool` default `False`
+            If True, will log-in at organization level.
 
     OBS:
         In case all parameters are `None`, pycarol will try yo find their values in the environment variables.
@@ -65,37 +75,31 @@ class Carol:
     """
 
     def __init__(self, domain=None, app_name=None, auth=None, connector_id=None, port=443, verbose=False,
-                 organization=None, environment=None, host=None):
+                 organization=None, environment=None, host=None, user=None, password=None, api_key=None, org_level=False):
+
+        self.connector_id = connector_id
 
         if auth is None:
-            carol_user = os.getenv('CAROLUSER')
-            carol_pw = os.getenv('CAROLPWD')
+            auth = self.__get_auth(connector_id=connector_id, username=user, password=password, app_oauth=api_key)
 
-            if carol_user and carol_pw:
-                auth = PwdAuth(user=carol_user, password=carol_pw)
-            else:
-                auth_token = os.getenv('CAROLAPPOAUTH')
-                connector_id = os.getenv('CAROLCONNECTORID')
-                auth = ApiKeyAuth(auth_token)
-            if auth is None:
-                raise ValueError("either `auth` method or pycarol env variables must be set.")
-
-        if domain is None:
+        if (domain is None) and (not org_level):
             domain = os.getenv('CAROLTENANT')
             if domain is None:
                 raise ValueError("`domain` must be set.")
 
+        if org_level:
+            domain = None
+
         if app_name is None:
             app_name = os.getenv('CAROLAPPNAME', ' ')
 
-
-        if connector_id is None:
+        if self.connector_id is None:
             if auth.connector_id is None:
-                connector_id = __CONNECTOR_PYCAROL__
+                self.connector_id = __CONNECTOR_PYCAROL__
             else:
-                connector_id = auth.connector_id
+                self.connector_id = auth.connector_id
 
-        if domain is None or app_name is None or auth is None:
+        if (domain is None and not org_level) or app_name is None or auth is None:
             raise ValueError("domain, app_name and auth must be specified as parameters, either " +
                              "in the environment variables CAROLTENANT, CAROLAPPNAME, CAROLAPPOAUTH" +
                              " or CAROLUSER+CAROLPWD and  CAROLCONNECTORID")
@@ -113,19 +117,61 @@ class Carol:
         self.host = self._set_host(domain=self.domain, organization=self.organization,
                                    environment=self.environment, host=host)
         self._tenant = None
-        self.connector_id = connector_id
+        self._current_user = None
         self.auth = auth
         self.auth.set_connector_id(self.connector_id)
+        self.session = None
+        self._is_org_level = org_level
+
         self.auth.login(self)
         self.response = None
 
         self.org = None
 
+    def __get_auth(self, connector_id=None, username=None, password=None, app_oauth=None):
+
+        if username is None:
+            username = os.getenv('CAROLUSER')
+
+        if password is None:
+            password = os.getenv('CAROLPWD')
+
+        if username and password:
+            return PwdAuth(user=username, password=password)
+            
+        if app_oauth is None:
+            app_oauth = os.getenv('CAROLAPPOAUTH')
+        
+        if connector_id is None:
+            connector_id = os.getenv('CAROLCONNECTORID', __CONNECTOR_PYCAROL__)
+            self.connector_id = connector_id
+        
+        if app_oauth and connector_id:
+            return ApiKeyAuth(app_oauth)
+
+        raise ValueError("either `auth` or `username/password` or `api_key` or pycarol env variables must be set.")
+
+    @property
+    def current_user(self):
+        """
+        Returns the current user.
+        
+        Args:
+            force: `bool` default `False`.
+                If True will force the request to fetch current user.
+        """
+
+        if self._current_user is None:
+            if self._is_org_level:
+                self._current_user = self.call_api('v2/orgUsers/current', )
+            else:
+                self._current_user = self.call_api('v2/users/current', )
+        return self._current_user
 
 
     @property
     def tenant(self):
-        if self._tenant is None:
+        if self._tenant is None and not self._is_org_level:
             self._tenant = Tenant(self).get_tenant_by_domain(self.domain)
         return self._tenant
 
@@ -161,7 +207,6 @@ class Carol:
             return f"{organization}.{environment}"
         else:
             return f"{domain}.{environment}"
-        pass
 
     @staticmethod
     def _retry_session(retries=5, session=None, backoff_factor=0.5, status_forcelist=(500, 502, 503, 504, 524),
@@ -207,7 +252,7 @@ class Carol:
     def call_api(self, path, method=None, data=None, auth=True, params=None, content_type='application/json', retries=8,
                  session=None, backoff_factor=0.5, status_forcelist=(502, 503, 504, 524), downloadable=False,
                  method_whitelist=frozenset(['HEAD', 'TRACE', 'GET', 'PUT', 'OPTIONS', 'DELETE', 'POST']), errors='raise',
-                 extra_headers=None, files=None,
+                 extra_headers=None, files=None, prefix_path='/api/',
                  **kwds):
         """
         This method handles all the API calls.
@@ -249,6 +294,8 @@ class Carol:
                 extra headers to be sent.
             files: `dict` default `None`
                 Used when uploading files to carol. This will be sent to :class: `requests.request`
+            prefix_path: `str` default `/api/`
+                Prefix path to be used to create the final url 'https://{self.host}:{self.port}{prefix_path}{path}.
             kwds: `dict` default `None`
                 Extra parameters to be sent to :class: `requests.request`
 
@@ -256,9 +303,11 @@ class Carol:
             Dict with API response.
 
         """
+        if session is not None:
+            self.session = session
 
         extra_headers = extra_headers or {}
-        url = f'https://{self.host}:{self.port}/api/{path}'
+        url = f'https://{self.host}:{self.port}{prefix_path}{path}'
 
         if method is None:
             if data is None:
@@ -291,11 +340,12 @@ class Carol:
 
         __count = 0
         while True:
-            if session is None:
-                session = self._retry_session(retries=retries, session=session, backoff_factor=backoff_factor,
-                                              status_forcelist=status_forcelist, method_whitelist=method_whitelist)
+            self.session = self._retry_session(
+                retries=retries, session=self.session, backoff_factor=backoff_factor,
+                status_forcelist=status_forcelist, method_whitelist=method_whitelist
+                )
 
-            response = session.request(method=method, url=url, data=data, json=data_json,
+            response = self.session.request(method=method, url=url, data=data, json=data_json,
                                        headers=headers, params=params, files=files, **kwds)
 
             if self.verbose:
@@ -324,21 +374,29 @@ class Carol:
                     continue
                 else:
                     raise Exception('Too many retries to refresh token.\n', response.text, response.status_code)
-            elif (response.status_code == 404):
+            elif response.status_code == 404:
                 raise CarolApiResponseException(response.text, response.status_code)
 
             raise Exception(response.text, response.status_code)
 
-    def issue_api_key(self):
+    def issue_api_key(self, connector_id=None):
         """
         Create an API key for a given connector.
 
+        Args:
+
+            connector_id: `str` default `None`.
+                Connector ID to be used when creating the APIkey
+
         Returns: `dict`
             Dictionary with the API key.
 
         """
+
+        if connector_id is None:
+            connector_id = self.connector_id
         resp = self.call_api('v2/apiKey/issue', data={
-            'connectorId': self.connector_id
+            'connectorId': connector_id
         }, content_type='application/x-www-form-urlencoded')
         return resp
 
@@ -385,30 +443,20 @@ class Carol:
 
         return resp
 
-    def copy_token(self):
-        """
-        Copy token to clipboard
-
-        Returns:
-            None
-        """
-
-        import pyperclip
-        if isinstance(self.auth, PwdAuth):
-            token = self.auth._token.access_token
-            pyperclip.copy(token)
-            print("Copied auth token to clipboard: " + token)
-        elif isinstance(self.auth, ApiKeyAuth):
-            token = self.auth.api_key
-            pyperclip.copy(token)
-            print("Copied API Key to clipboard: " + token)
-        else:
-            raise Exception("Auth object not set. Can't fetch token.")
-
     def switch_org_level(self):
+        """
+        Switch organization level.
 
+        """
+        if self._is_org_level:
+            warnings.warn(
+                "already in org level.",
+                UserWarning, stacklevel=3
+            )
+            return
         org = self._current_org()
         self.auth.switch_org_context(org['mdmId'])
+        self._is_org_level = True
 
     def switch_environment(self, env_name=None, env_id=None, app_name=None, org_name=None, org_id=None):
         """
@@ -452,7 +500,8 @@ class Carol:
 
         if (org_id is not None and org_id != current['org_id']) or (org_name is not None and org_name != current['org_name'] ):
             # Switch to org context.
-            self.switch_org_level()
+            if not current['org_level']:
+                self.switch_org_level()
             if org_id is None:
                 org_id = Organization(self).get_organization_info(org_name)['mdmId']
             # Switch org.
@@ -460,14 +509,16 @@ class Carol:
 
         if env_name:
             env_id = Tenant(self).get_tenant_by_domain(env_name)['mdmId']
-        elif env_id is None:
-            raise ValueError('Either `env_name` or `env_id` must be set.')
-
-        self.auth.switch_context(env_id=env_id)
+        if env_id is not None:
+            self.auth.switch_context(env_id=env_id)
+            self._tenant = self._current_env()
+            self._is_org_level = False
+        else:
+            self._tenant = None
+            self._is_org_level = True
 
         self.domain = env_name
         self.app_name = app_name # TODO: Today we cannot use CDS without a valid app name.
-        self._tenant = self._current_env()
         self.organization = self._current_org()['mdmName']
 
         self.host = self._set_host(domain=self.domain, organization=self.organization,
@@ -510,8 +561,6 @@ class Carol:
 
         if env_name:
             env_id = Tenant(self).get_tenant_by_domain(env_name)['mdmId']
-        elif env_id is None:
-            raise ValueError('Either `env_name` or `env_id` must be set.')
 
         class SwitchContext(object):
 
@@ -537,7 +586,7 @@ class Carol:
 
 
     def _current_env(self):
-        return self.call_api('v1/tenants/current')
+        return self.call_api('v1/tenants/current', errors='ignore')
 
     def _current_org(self):
         return self.call_api('v1/organizations/current')
@@ -571,9 +620,26 @@ class Carol:
         if level.lower() == 'org' or level.lower() == 'all':
             org = self._current_org()
 
+        if env.get('mdmName') is None:
+            self._is_org_level = True
+        else:
+            self._is_org_level = False
+
         return {
             "env_name": env.get('mdmName'),
             "env_id": env.get('mdmId'),
             "org_name": org.get('mdmName'),
             "org_id": org.get('mdmId'),
+            "org_level": self._is_org_level
         }
+
+    def get_tenants_for_user(self):
+        """
+        Get all tenants for the current user.
+
+        Returns: `list`
+            List of tenants.
+
+        """
+
+        return self.call_api("v1/users/assignedTenantsForCurrentUser", method='GET')
