@@ -8,7 +8,7 @@ from .storage import Storage
 from .utils.importers import _import_dask, _import_pandas
 from .utils import async_helpers
 from .utils.miscellaneous import stream_data
-from . import _CAROL_METADATA_STAGING, _NEEDED_FOR_MERGE
+from . import _CAROL_METADATA_STAGING, _NEEDED_FOR_MERGE, _CAROL_METADATA_UNTIE_STAGING
 from .utils.miscellaneous import drop_duplicated_parquet, drop_duplicated_parquet_dask
 from .utils.deprecation_msgs import _deprecation_msgs
 
@@ -121,12 +121,18 @@ class Staging:
         elif isinstance(data, list):
             data_size = len(data)
             _sample_json = data[0]
+        elif isinstance(data, dict):
+            data = [data]
+            data_size = len(data)
+            _sample_json = data[0]
         else:
             import pandas as pd
             if isinstance(data, pd.DataFrame):
                 is_df = True
                 data_size = data.shape[0]
                 _sample_json = data.iloc[0].to_json(date_format='iso')
+            else:
+                raise ValueError('`data` should be either a stringfied json, a list of dictionaries, a dictionary or a pd.DataGrame ')
 
 
         if (not isinstance(data, list)) and (not is_df):
@@ -525,9 +531,9 @@ class Staging:
 
         if merge_records:
             if (not return_dask_graph) or (backend == 'pandas'):
-                d = drop_duplicated_parquet(d)
+                d = drop_duplicated_parquet(d, untie_field=_CAROL_METADATA_UNTIE_STAGING)
             else:
-                d = drop_duplicated_parquet_dask(d)
+                d = drop_duplicated_parquet_dask(d, untie_field=_CAROL_METADATA_UNTIE_STAGING)
 
         if not return_metadata:
             to_drop = set(_meta_cols).intersection(set(d.columns))
@@ -725,3 +731,43 @@ class Staging:
         if publish:
             url = f"v1/connectors/{connector_id}/entityMappings/{_mapping_id}/publish"
             self.carol.call_api(url, method='POST')
+
+
+    def generate_SQL_tables(self, connector_id=None, connector_name=None):
+        """ Generates SQL tables from a connector.
+
+        The tables created will be named:
+        `stg_{connector_id}_{staging_name}`. 
+
+        Args:
+            connector_id (str, optional): Connector ID. Defaults to None.
+            connector_name (str, optional): Connector name. Defaults to None.
+
+        Returns:
+            dict: Carol's response
+        
+
+        """
+        if connector_name:
+            connector_id = self._connector_by_name(connector_name)
+        elif connector_id is None:
+            raise ValueError("Either `connector_id` or `connector_name` should be set") 
+
+        params = {"connectorId": connector_id}
+        return self.carol.call_api(path='v3/staging/generateSqlTables', method='POST', params=params, )
+
+
+    def remove_SQL_tables(self, connector_id=None, connector_name=None):
+        """Remove all SQL tables from a connector
+
+        Args:
+            connector_id (str, optional): Connector ID. Defaults to None.
+            connector_name (str, optional): Connector name. Defaults to None.
+
+        Returns:
+            dict: Carol's response
+        
+
+        """
+
+        raise NotImplementedError("API not inplemented in Carol yet. Talk for support.")
\ No newline at end of file
