@@ -5,19 +5,26 @@ from pathlib import Path
 _FILE_MARKER = '<files>'
 
 
-def drop_duplicated_parquet_dask(d):
+def drop_duplicated_parquet_dask(d, untie_field='mdmCounterForEntity'):
     """
     Merge updates and delete records from the parquet files in CDS.
 
     Args:
         d: dask DataFrame
+        untie_field: str
+            Field to be used to untie records with the same `mdmId`. 
 
     Returns:
         dask DataFrame
 
     """
 
-    d = d.set_index('mdmCounterForEntity', ) \
+    if untie_field not in d.columns:
+        #Use the standard one. 
+        untie_field = 'mdmCounterForEntity'
+
+
+    d = d.set_index(untie_field, ) \
         .drop_duplicates(subset='mdmId', keep='last') \
         .reset_index()
     if 'mdmDeleted' in d.columns:
@@ -27,19 +34,25 @@ def drop_duplicated_parquet_dask(d):
     return d
 
 
-def drop_duplicated_parquet(d):
+def drop_duplicated_parquet(d, untie_field='mdmCounterForEntity'):
     """
     Merge updates and delete records from the parquet files in CDS.
 
     Args:
         d: pd.DataFrame
+            Dataframe
+        untie_field: str
+            Field to be used to untie records with the same `mdmId`. 
 
     Returns:
         pd.DataFrame
 
     """
 
-    d = d.sort_values('mdmCounterForEntity').reset_index(drop=True).drop_duplicates(subset='mdmId', keep='last')
+    if untie_field not in d.columns:
+        untie_field = 'mdmCounterForEntity'
+
+    d = d.sort_values(untie_field).reset_index(drop=True).drop_duplicates(subset='mdmId', keep='last')
     if 'mdmDeleted' in d.columns:
         d['mdmDeleted'] = d['mdmDeleted'].fillna(False)
         d = d[~d['mdmDeleted']]
@@ -174,3 +187,40 @@ def zip_folder(path_to_zip):
             z.write(f, arcname=f.relative_to(path_to_zip.parents[0]))
 
     return zip_file
+
+def find_keys(node, kv):
+    """Find values of a key in a nested dictionary.
+
+    Args:
+
+        node (dict): Dictionary to search.
+        kv (str): key to search.
+
+    Yields:
+        str: values.
+
+    """
+    if isinstance(node, list):
+        for i in node:
+            yield from find_keys(i, kv)
+    elif isinstance(node, dict):
+        if kv in node:
+            yield node[kv]
+        for j in node.values():
+            yield from find_keys(j, kv)
+                
+def unroll_list(l):
+    """Unroll a list of lists.
+
+    Args:
+        l (list): list of list to unroll.
+
+    Yields:
+        list: unrolled list.
+    """
+
+    if isinstance(l, list):
+        for i in l:
+            yield from unroll_list(i)
+    else:
+        yield l
\ No newline at end of file
